import { RemoveScrollUIType, IRemoveScrollSelfProps } from './types';
export type { IRemoveScrollSelfProps, RemoveScrollUIType };
/**
 * Removes scrollbar from the page and contain the scroll within the Lock
 */
declare const RemoveScroll: RemoveScrollUIType;
export { RemoveScroll };
